@@ -1,23 +1,24 @@
 /* Jittery structured language example: main.
 
-   Copyright (C) 2017, 2019, 2020 Luca Saiu
+   Copyright (C) 2017, 2019, 2020, 2021 Luca Saiu
+   Updated in 2022 by Luca Saiu
    Written by Luca Saiu
 
    This file is part of the Jitter structured-language example, distributed
-   along with Jitter under the same license.
+   along with GNU Jitter under the same license.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <http://www.gnu.org/licenses/>. */
 
 
 #include <stdio.h>
@@ -100,20 +101,34 @@ structured_help (void)
 
   printf ("Usage: %s [OPTION...] FILE.structured\n", structured_program_name);
   printf ("   or: %s [OPTION...] -\n", structured_program_name);
-  printf ("Run a structured-language program on a Jittery VM, using the\n");
-  printf ("%s dispatch %s.\n", c->dispatch_human_readable,
-          jitter_vm_instrumentation_to_string (instrumentation));
+  printf ("Run a structured-language program on a Jittery VM, either\n"
+          "stack-based or register-based according to the options,\n"
+          "using the");
+  const char *instrumentation_string
+    = jitter_vm_instrumentation_to_string (instrumentation);
+  bool instrumentation_empty = instrumentation_string [0] == '\0';
+  printf ("%c%s dispatch%s%s.\n",
+          instrumentation_empty ? ' ' : '\n',
+          c->dispatch_human_readable,
+          instrumentation_empty ? "" : " with ",
+          instrumentation_string);
 
   structured_help_section ("Debugging options");
   printf ("      --disassemble                print hardware machine instructions\n");
   printf ("      --cross-disassemble          use the cross-disassembler rather than\n");
   printf ("                                   the native disassembler; also enable\n");
   printf ("                                   disassembly as per --disassemble\n");
+  printf ("      --print-defects=WHAT         print information about defective\n");
+  printf ("                                   specialised instructions and their\n");
+  printf ("                                   replacements.  WHAT must be one of:\n");
+  printf ("                                   \"summary\", \"list\", \"replacements\",\n");
+  printf ("                                   \"no\"\n");
   printf ("      --profile-specialized        print profiling information for\n");
   printf ("                                   specialized instructions %s\n",
           (instrumentation != jitter_vm_instrumentation_none
            ? ""
-           : " if available (disabled: recompile with -DPROFILE_COUNT=1 or -DPROFILE_SAMPLE=1)"));
+           : "\n                                   "
+             "if available (disabled: recompile with -DPROFILE_COUNT=1 or -DPROFILE_SAMPLE=1)"));
   printf ("      --profile-unspecialized      like --profile-specialized, for\n");
   printf ("                                   unspecialized instructions\n");
   printf ("      --print-locations            print the mapping between VM structures\n");
@@ -122,6 +137,10 @@ structured_help (void)
   printf ("      --dry-run                    do not actually run the program\n");
   printf ("      --print-routine, --print     print VM instructions\n");
   printf ("      --no-dry-run                 run the program (default)\n");
+  printf ("      --no-print-defects           do not print information about defective\n");
+  printf ("                                   specialized instructions and\n");
+  printf ("                                   replacements; (default and functionally\n");
+  printf ("                                   equivalent to --print-defects=no)\n");
   printf ("      --no-profile-specialized     omit profiling information for specialized\n");
   printf ("                                   instructions (default)\n");
   printf ("      --no-profile-unspecialized   omit profiling information for unspecialized\n");
@@ -156,7 +175,10 @@ structured_help (void)
   printf ("An \"--\" argument terminates option processing.\n");
 
   printf ("\n");
-  printf ("Report bugs to " JITTER_PACKAGE_BUGREPORT ".\n");
+  printf (JITTER_PACKAGE_NAME " home page: <" JITTER_PACKAGE_URL ">.\n");
+  printf ("\n");
+  printf ("Report bugs to <" JITTER_PACKAGE_BUGREPORT ">.\n");
+  printf ("General help using GNU software: <https://www.gnu.org/gethelp/>.\n");
 
   exit (EXIT_SUCCESS);
 }
@@ -169,14 +191,14 @@ structured_version (void)
   const char *instrumentation
     = jitter_vm_instrumentation_to_string (c->instrumentation);
 
-  printf ("structured, %s%s%s dispatch ("
+  printf ("Structured language executor with %s%s%s dispatch ("
           JITTER_PACKAGE_NAME ") " JITTER_PACKAGE_VERSION "\n",
           instrumentation,
           (strlen (instrumentation) > 0 ? ", " : ""),
           c->dispatch_human_readable);
-  printf ("Copyright (C) 2021 Luca Saiu.\n"
-          "Jitter comes with ABSOLUTELY NO WARRANTY.\n"
-          "You may redistribute copies of Jitter under the terms of the GNU\n"
+  printf ("Copyright (C) 2022 Luca Saiu.\n"
+          "GNU Jitter comes with ABSOLUTELY NO WARRANTY.\n"
+          "You may redistribute copies of GNU Jitter under the terms of the GNU\n"
           "General Public License, version 3 or any later version published\n"
           "by the Free Software Foundation.  For more information see the\n"
           "file named COPYING.\n"
@@ -202,6 +224,15 @@ enum structured_code_generator
     structured_code_generator_register
   };
 
+/* What to print about defective specialised instructions. */
+enum structured_print_defect_what
+  {
+    structured_print_defect_what_summary,
+    structured_print_defect_what_list,
+    structured_print_defect_what_replacements,
+    structured_print_defect_what_no
+  };
+
 /* The state encoded in a user command line. */
 struct structured_command_line
 {
@@ -216,6 +247,10 @@ struct structured_command_line
   /* True iff we should disassemble the VM routine. */
   bool disassemble;
 
+  /* True iff we should print information about defective specialised
+     instructions and their replacements. */
+  enum structured_print_defect_what print_defects;
+
   /* True iff we should print profiling information, respectively for
      specialised and unspecialised instructions. */
   bool profile_specialized;
@@ -252,6 +287,7 @@ structured_initialize_command_line (struct structured_command_line *cl)
   cl->print = false;
   cl->cross_disassemble = false;
   cl->disassemble = false;
+  cl->print_defects = structured_print_defect_what_no;
   cl->profile_specialized = false;
   cl->profile_unspecialized = false;
   cl->print_locations = false;
@@ -310,10 +346,20 @@ structured_parse_command_line (struct structured_command_line *cl,
           cl->cross_disassemble = true;
           cl->disassemble = true;
         }
+      else if (handle_options && ! strcmp (arg, "--print-defects=summary"))
+        cl->print_defects = structured_print_defect_what_summary;
+      else if (handle_options && ! strcmp (arg, "--print-defects=list"))
+        cl->print_defects = structured_print_defect_what_list;
+      else if (handle_options && ! strcmp (arg, "--print-defects=replacements"))
+        cl->print_defects = structured_print_defect_what_replacements;
+      else if (handle_options && ! strcmp (arg, "--print-defects=no"))
+        cl->print_defects = structured_print_defect_what_no;
       else if (handle_options && ! strcmp (arg, "--profile-specialized"))
         cl->profile_specialized = true;
       else if (handle_options && ! strcmp (arg, "--profile-unspecialized"))
         cl->profile_unspecialized = true;
+      else if (handle_options && ! strcmp (arg, "--no-print-defects"))
+        cl->print_defects = structured_print_defect_what_no;
       else if (handle_options && ! strcmp (arg, "--no-profile-unspecialized"))
         cl->profile_unspecialized = false;
       else if (handle_options && ! strcmp (arg, "--no-profile-specialized"))
@@ -377,6 +423,22 @@ structured_parse_command_line (struct structured_command_line *cl,
 /* Execute what the command line says.
  * ************************************************************************** */
 
+/* Print a heading.  This is useful for defects. */
+static void
+structured_print_heading (jitter_print_context ctx, const char *heading)
+{
+  jitter_print_begin_class (ctx, "structuredvm-comment");
+  if (structuredvm_vm->defect_no == 0)
+    jitter_print_char_star (ctx, "No defects.");
+  else
+    {
+      jitter_print_char_star (ctx, heading);
+      jitter_print_char_star (ctx, ":");
+    }
+  jitter_print_end_class (ctx);
+  jitter_print_char_star (ctx, "\n");
+}
+
 /* Do what the pointed command line data structure says. */
 static void
 structured_work (struct structured_command_line *cl)
@@ -439,6 +501,29 @@ structured_work (struct structured_command_line *cl)
      structuredvm_make_executable_routine.  However the unified API makes this
      automatic. */
 
+  /* Print defect information if the user asked to */
+  switch (cl->print_defects)
+    {
+    case structured_print_defect_what_summary:
+      /* No heading: this is a compact format. */
+      structuredvm_defect_print_summary (ctx);
+      break;
+    case structured_print_defect_what_list:
+      structured_print_heading (ctx, "Defective specialised instructions");
+      structuredvm_defect_print (ctx, 0);
+      break;
+    case structured_print_defect_what_replacements:
+      structured_print_heading
+         (ctx, "Defective specialised instruction replacements");
+      structuredvm_defect_print_replacement_table (ctx, 0);
+      break;
+    case structured_print_defect_what_no:
+      /* Do nothing. */
+      break;
+    default:
+      jitter_fatal ("invalid cl->print_defects value: this should never happen");
+    }
+
   /* Print and/or disassemble the routine as requested. */
   if (cl->print)
     structuredvm_routine_print (ctx, vmr);
