@@ -1,6 +1,6 @@
 /* pkl-anal.c - Analysis phases for the poke compiler.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -222,7 +222,8 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_type_struct)
                         PKL_AST_IDENTIFIER_POINTER (tname)))
             {
               PKL_ERROR (PKL_AST_LOC (u),
-                         "duplicated element name in struct type spec");
+                         "duplicated struct element '%s'",
+                         PKL_AST_IDENTIFIER_POINTER (uname));
               PKL_ANAL_PAYLOAD->errors++;
               PKL_PASS_ERROR;
             }
@@ -293,7 +294,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_funcall)
   if (some_named && some_unnamed)
     {
       PKL_ERROR (PKL_AST_LOC (funcall),
-                 "mixed named and not-named arguments not allowed in funcall");
+                 "found named and not-named arguments mixed in funcall");
       PKL_ANAL_PAYLOAD->errors++;
       PKL_PASS_ERROR;
     }
@@ -344,14 +345,18 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_func)
 {
   pkl_ast_node func = PKL_PASS_NODE;
   pkl_ast_node fa;
+  int in_opt_args = 0;
 
-  for (fa = PKL_AST_FUNC_FIRST_OPT_ARG (func);
+  for (fa = PKL_AST_FUNC_ARGS (func);
        fa;
        fa = PKL_AST_CHAIN (fa))
     {
+      if (fa == PKL_AST_FUNC_FIRST_OPT_ARG (func))
+        in_opt_args = 1;
+
       /* All optional formal arguments in a function specifier should
          be at the end of the arguments list.  */
-      if (!PKL_AST_FUNC_ARG_INITIAL (fa))
+      if (in_opt_args && !PKL_AST_FUNC_ARG_INITIAL (fa))
         {
           PKL_ERROR (PKL_AST_LOC (fa),
                      "non-optional argument after optional arguments");
@@ -361,8 +366,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_func)
 
       /* If there is a vararg argument, it should be at the end of the
          list of arguments.  Also, it should be unique.  */
-      if (PKL_AST_FUNC_ARG_VARARG (fa) == 1
-          && PKL_AST_CHAIN (fa) != NULL)
+      if (PKL_AST_FUNC_ARG_VARARG (fa) == 1 && PKL_AST_CHAIN (fa) != NULL)
         {
           PKL_ERROR (PKL_AST_LOC (fa),
                      "vararg argument should be the last argument");
@@ -389,7 +393,8 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_type_function)
        arg = PKL_AST_CHAIN (arg))
     {
       if (PKL_AST_FUNC_TYPE_ARG_VARARG (arg)
-          && PKL_AST_CHAIN (arg) != NULL)
+          && PKL_AST_CHAIN (arg) != NULL
+          && PKL_AST_LOC_VALID (PKL_AST_LOC (arg)))
         {
           PKL_ERROR (PKL_AST_LOC (arg),
                      "vararg argument should be the last argument");
@@ -449,6 +454,23 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_return_stmt)
 }
 PKL_PHASE_END_HANDLER
 
+/* Arrays of `void' are not allowed.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_type_array)
+{
+  pkl_ast_node type = PKL_PASS_NODE;
+  pkl_ast_node etype = PKL_AST_TYPE_A_ETYPE (type);
+
+  if (PKL_AST_TYPE_CODE (etype) == PKL_TYPE_VOID)
+    {
+      PKL_ERROR (PKL_AST_LOC (etype),
+                 "arrays of type `void' are not allowed");
+      PKL_ANAL_PAYLOAD->errors++;
+      PKL_PASS_ERROR;
+    }
+}
+PKL_PHASE_END_HANDLER
+
 /* If the unit in an offset type specifier is specified using an
    integral constant, this constant should be bigger than zero.  */
 
@@ -651,11 +673,12 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_cons)
   switch (PKL_AST_TYPE_CODE (cons_type))
     {
     case PKL_TYPE_STRUCT:
-      /* Struct constructors accept exactly one argument.  */
+      /* Struct constructors accept exactly one argument.  This is
+         enforced by the syntax.  */
       if (pkl_ast_chain_length (cons_value) != 1)
         {
-          PKL_ERROR (PKL_AST_LOC (cons),
-                     "struct constructor requires exactly one argument");
+          PKL_ICE (PKL_AST_LOC (cons),
+                   "struct constructor requires exactly one argument");
           PKL_ANAL_PAYLOAD->errors++;
           PKL_PASS_ERROR;
         }
@@ -699,6 +722,7 @@ struct pkl_phase pkl_phase_anal1 =
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_STRUCT, pkl_anal1_ps_type_struct),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_FUNCTION, pkl_anal1_ps_type_function),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_OFFSET, pkl_anal1_ps_type_offset),
+   PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_ARRAY, pkl_anal1_ps_type_array),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_SL, pkl_anal1_ps_op_sl),
    PKL_PHASE_PS_DEFAULT_HANDLER (pkl_anal_ps_default),
   };
@@ -816,6 +840,8 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_funcall)
       && PKL_PASS_PARENT
       && PKL_AST_CODE (PKL_PASS_PARENT) != PKL_AST_EXP_STMT)
     {
+      /* Note that this error is catched earlier in
+         pkl_typify1_ps_funcall */
       PKL_ERROR (PKL_AST_LOC (funcall_function),
                  "call to void function in expression");
       PKL_ANAL_PAYLOAD->errors++;
@@ -836,8 +862,18 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_struct_type_field)
       && (PKL_AST_TYPE_CODE (type) == PKL_TYPE_STRUCT
           || PKL_AST_TYPE_CODE (type) == PKL_TYPE_ARRAY)
       && PKL_AST_TYPE_NAME (type))
-    PKL_WARNING (PKL_AST_LOC (field),
-                 "useless endianness annotation in field");
+    {
+      const char *msg = "useless endianness annotation in field";
+
+      if (pkl_error_on_warning (PKL_PASS_COMPILER))
+        {
+          PKL_ERROR (PKL_AST_LOC (field), msg);
+          PKL_ANAL_PAYLOAD->errors ++;
+          PKL_PASS_ERROR;
+        }
+      else
+        PKL_WARNING (PKL_AST_LOC (field), msg);
+    }
 }
 PKL_PHASE_END_HANDLER
 
@@ -879,8 +915,16 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_type_struct)
 
           if (last_unconditional_alternative)
             {
-              PKL_WARNING (PKL_AST_LOC (t),
-                           "unreachable alternative in union");
+              const char *msg = "unreachable alternative in union";
+
+              if (pkl_error_on_warning (PKL_PASS_COMPILER))
+                {
+                  PKL_ERROR (PKL_AST_LOC (t), msg);
+                  PKL_ANAL_PAYLOAD->errors ++;
+                  PKL_PASS_ERROR;
+                }
+              else
+                PKL_WARNING (PKL_AST_LOC (t), msg);
               break;
             }
 
@@ -897,8 +941,16 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_type_struct)
               && PKL_AST_CODE (constraint) == PKL_AST_INTEGER
               && PKL_AST_INTEGER_VALUE (constraint) == 0)
             {
-              PKL_WARNING (PKL_AST_LOC (t),
-                           "unreachable alternative in union");
+              const char *msg = "unreachable alternative in union";
+
+              if (pkl_error_on_warning (PKL_PASS_COMPILER))
+                {
+                  PKL_ERROR (PKL_AST_LOC (t), msg);
+                  PKL_ANAL_PAYLOAD->errors ++;
+                  PKL_PASS_ERROR;
+                }
+              else
+                PKL_WARNING (PKL_AST_LOC (t), msg);
               break;
             }
         }
@@ -906,37 +958,6 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_type_struct)
 }
 PKL_PHASE_END_HANDLER
 
-/* The indexes in array initializers shall be constant.  */
-
-PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_array)
-{
-  pkl_ast_node array = PKL_PASS_NODE;
-  pkl_ast_node initializers
-    = PKL_AST_ARRAY_INITIALIZERS (array);
-  pkl_ast_node initializer;
-
-  for (initializer = initializers;
-       initializer;
-       initializer = PKL_AST_CHAIN (initializer))
-    {
-      pkl_ast_node index
-        = PKL_AST_ARRAY_INITIALIZER_INDEX (initializer);
-
-      /* pkl_trans1_ps_array should install indexes in all
-         initializers.  */
-      assert (index);
-
-      if (PKL_AST_CODE (index) != PKL_AST_INTEGER)
-        {
-          PKL_ERROR (PKL_AST_LOC (index),
-                     "indexes in array initializers shall be constant");
-          PKL_ANAL_PAYLOAD->errors++;
-          PKL_PASS_ERROR;
-        }
-    }
-}
-PKL_PHASE_END_HANDLER
-
 struct pkl_phase pkl_phase_anal2 =
   {
    PKL_PHASE_PS_HANDLER (PKL_AST_SRC, pkl_anal_ps_src),
@@ -949,7 +970,6 @@ struct pkl_phase pkl_phase_anal2 =
    PKL_PHASE_PS_HANDLER (PKL_AST_RETURN_STMT, pkl_anal2_ps_return_stmt),
    PKL_PHASE_PS_HANDLER (PKL_AST_FUNCALL, pkl_anal2_ps_funcall),
    PKL_PHASE_PS_HANDLER (PKL_AST_STRUCT_TYPE_FIELD, pkl_anal2_ps_struct_type_field),
-   PKL_PHASE_PS_HANDLER (PKL_AST_ARRAY, pkl_anal2_ps_array),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_STRUCT, pkl_anal2_ps_type_struct),
    PKL_PHASE_PS_DEFAULT_HANDLER (pkl_anal_ps_default),
   };
@@ -990,9 +1010,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_analf_ps_ass_stmt)
 PKL_PHASE_END_HANDLER
 
 /* Make sure that the argument to an incrdecr operator is of the right
-   kind, i.e. it is a valid lvalue.
-
-   XXX this shouldn't be necessary due to the check in ass_stmt!  */
+   kind, i.e. it is a valid lvalue.  */
 
 PKL_PHASE_BEGIN_HANDLER (pkl_analf_ps_incrdecr)
 {
@@ -1006,6 +1024,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_analf_ps_incrdecr)
 
       if (!pkl_ast_lvalue_p (incrdecr_exp))
         {
+          /* Note that this is already checked in ass_stmt.  */
           PKL_ERROR (PKL_AST_LOC (incrdecr_exp),
                      "invalid operand to %s%s",
                      incrdecr_order == PKL_AST_ORDER_PRE ? "pre" : "post",
